:
# RCSid:
#	$Id: funcs.sh,v 1.52 2022/10/05 19:25:17 sjg Exp $
#
#	@(#) Copyright (c) 1994 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

_FUNCS_SH=:

# we need to avoid redefining source() since
# some shells blow up if we redefined a func while it is
# in use on the stack (as it were).
$_SOURCE_SH . ${RC_D:-/etc/rc_d}/source.sh

# We want to use local if we can
# if isposix-shell.sh has been sourced isPOSIX_SHELL will be set
case "$isPOSIX_SHELL$local" in
"") if (echo ${PATH%:*}) > /dev/null 2>&1; then
        local=local
    else
        local=:
    fi
    ;;
esac

# cannot rely on these being consistent.
case `echo -- .` in -*) echo__=;; *) echo__=--;; esac
case `echo -n .` in -n*) echo_n=; echo_c="\c";; *) echo_n=-n; esac
# for older scripts
N=$echo_n; C="$echo_c"

# provide consistent behavior
Echo() {
        eval $local __o __c
	__o=
	__c=
	while :
	do
	        : 1="$1"
		case "$1" in
		--) shift; __o="$__o $echo__"; break;;
		-n) shift; __o="$__o $echo_n"; __c="$echo_c";;
		*) break;;
		esac
	done
	echo $__o "$@$__c"
}

OpChk() {
        eval $local __c __o __t
        __t=$1; shift
        __c=$1; shift
        __o=$1; shift
        case `$__c -$__o "${@:-/dev/null}" 2>&1` in
	*such*|"") eval "${__t}_$__o=-$__o";;
	esac
}

# this one doesn't always work (eg echo__="" and -e does something)
chk_test() {
	case `test $1 ${2:-/dev/null} 2>&1` in
	"")	echo $echo__ $1;;
	esac
}

OpChk test test e
OpChk test test s
OpChk test test h
OpChk test test L
# -h may be deprecated, prefer -L
test_L=${test_L:-$test_h}

Exists() {
        eval $local f _t
	case "$1" in
	-?) _t=$1; shift;;
	*) _t=-s;;
	esac
	for f in $*
	do
		test $_t $f || continue
		echo $f
                return 0
	done
        return 1
}

exists() {
        Exists "$@" > /dev/null
}

RC_D=${RC_D:-/etc/rc_d}

$_OS_SH source_rc $RC_D/os.sh

# use this if .pid file contains more than plain pid
# just set EXTRACT_PID=extract_pid
extract_pid() {
	sed -n -e 's,^[^0-9]*\([0-9][0-9]*\).*,\1,p' "$@"
}

get_pid() {
	case "$1" in
	[1-9]*)
		echo $1
		;;
	*/*.pid)
		test -s $1 && ${EXTRACT_PID:-cat} $1
		;;
	*)
		case "$1" in
		*.pid)	_pf=$1;;
		*)	_pf=$1.pid;;
		esac
		for d in `IFS=:; echo $2` /var/run /etc 
		do
			test -s $d/$_pf && { ${EXTRACT_PID:-cat} $d/$_pf; break; }
			test -s $d/.$_pf && { ${EXTRACT_PID:-cat} $d/.$_pf; break; }
		done
		;;
	esac
}

# we look for a pid file, and if not found, we try ps(1).

get_pids() {
	pids=
	
	# build up a list of pids or a regexp to find them
	re=
	for p in $*
	do
		pid=`get_pid $p $PID_PATH`
		
		test "$pid" && { pids="$pids $pid"; continue; }

		case "$re" in
		*.pid) continue;;
		esac
		test "$re" && re="$re|$p" || re=$p
	done
	
	if test "$re"; then
		case "$re" in
		*/*) re=`echo $re | sed 'y,/,.,'`;;
		esac
		# PS_AXC should be the equivalent of ps -axc
		# or ps -e under sysV
		ps=${PS:-ps}
		ps_opts=${PS_OPTS:-${PS_AXC:-"-axc"}}
		case "$ps_opts" in
		-ef) # sometimes we need full command line
		    pid_field='$2'
		    ;;
		*)  pid_field='$1';;
		esac
		if [ "$PS_FILE" ]; then
			# sometimes we know we will call this a lot,
			# we also need it if using -ef or equivalent
			[ -s $PS_FILE ] || $ps $ps_opts > $PS_FILE
			pids="$pids `awk "/$$/ { next } /$re/ { print $pid_field }" $PS_FILE`"
		else
			pids="$pids `$ps $ps_opts | awk "/$$/ { next } /$re/ { print $pid_field }"`"
		fi
	fi
	
	echo $pids
}

# like kill(1) but return 2 for no-such process
Kill() {
	# not a problem yet, but it won't be long before
	# someone starts internationalizing the error strings
	_m=`LANG=C LC_ALL=C kill "$@" 2>&1`
	_x=$?
	case "$_m" in
	*such*p*) _x=2;;
	esac
	return $_x
}

PidAlive() {
	Kill -0 "$@"
	[ $? != 2 ]
}

WaitForPid() {
	_wfp_t=60
	while :
	do
		case "$1" in
		-t) _wfp_t=$2; shift 2;;
		--) shift; break;;
		*) break;;
		esac
	done
	i=0
	while [ $i -lt $_wfp_t ]
	do
		PidAlive $* || break
		${TWDL:-:}
		sleep 1
		i=`expr $i + 1`
	done
	${TWDL:-:} " "
	PidAlive $*
}

# This logic is very common
# -k secs	wait "secs" for pids to die then kill -9
# -s secs	if we have to kill -9 sleep extra secs afterwards
# -t secs	wait "secs" for pids to die - after -k secs if set
# -v		be verbose about murder
# -[0-9]*	alternate initial signal
Zap() {
	_Zap_s=
	_Zap_k=
	_Zap_t=
	_Zap_v=:
	while :
	do
		case "$1" in
		-k) _Zap_k=$2; shift 2;;
		-s) _Zap_s=$2; shift 2;;
		-t) _Zap_t="-t $2"; shift 2;;
		-v) _Zap_v= ; shift;;
		-[1-9]*) _Zap_s=$1; shift;;
		--) shift; break;;
		*) break;;
		esac
	done
        if test "$1"; then
        	$ECHO Kill $_Zap_s $*
        	if test "$_Zap_k"; then
        		if WaitForPid -t $_Zap_k $*; then
        			for _Zap_p in $*
        			do
        				PidAlive $_Zap_p || continue
        				$_Zap_v echo "$_Zap_p: still alive! murdering it..."
        				$ECHO Kill -9 $_Zap_p
				done
        			# we just mudered something, 
        			# so we may need an extra delay...
        			${_Zap_t:+sleep} $_Zap_s
			fi
		fi
        	WaitForPid $_Zap_t $*
	fi
}

max_limit() {
	# Linux uses bash as /bin/sh and unlike
	# every other sh like shell it does not
	# handle -<x>H but does handle -H<x>
	# -H -<x> works on some but not all bash versions <grrr/>
	# so we have to make sure we strip any '-' from our args...
	for _ul in `IFS=" -"; echo $*`
	do
		_ulmax=`ulimit -H$_ul`
		ulimit -$_ul $_ulmax
	done
}

# each arg is a process name to stop.
zap() {
	pids=${pids:-`get_pids $*`}

	if test "$pids"; then
		_zap_k=
		case "$ZAP_SIG" in
		""|-15) test "x$NO_KILL9" = x || _zap_k="-k 16";;
		esac
		Zap $_zap_k -t 0 $ZAP_SIG $pids
	fi
}

_TTY=${_TTY:-`test -t 0 && tty`}; export _TTY

ask() { test "$_TTY" && { echo $echo_n "${2:-$1?} $echo_c" > $_TTY; read $1 < $_TTY; }; }

drop_last() {
	case "$2" in
	"")	return;;
	esac

	sep=$1
	var=$2
        joint=${3:-$sep}
	marker=${4:-#}

	case "$var" in
	*[${sep}]*)
		set -- `IFS="$sep"; echo $var$marker`
		out=$1; shift
		# BSD ash bug?
		[ "$out" ] || { out=$1; shift; }
		while :	
		do
			case "$1" in *"$marker") break;; esac
			out="$out$joint$1"; shift
		done
		echo $out
		;;
	esac
}

# this is as good as the real thing so just use it.
# Dirname dir
# behaves like real thing,
# Dirname dir anything
# tries to ensure result is absolute
Dirname() {
	case "$1" in
	*?/*)
		_d=
		case "$1" in
		/*) _d=/;;
		esac 
		_d="$_d`drop_last / $1`"
		case "$_d" in
		/*)	echo $_d;;
		*)	{ [ "$2" ] && (cd $_d 2>/dev/null && pwd); } || echo $_d;;
		esac
		;;
       /*)
	       echo /
	       ;;
	*)
		[ "$2" ] && pwd || echo .
		;;
	esac
}

_source ${RC_D:-/etc/rc_d}/runas.sh $Mydir/.. ""
